@@ -10,6 +10,8 @@ class TornadoClient(BaseClient):
     tornado.http.AsyncHTTPClient.
     """
 
+    __schemes__ = set(['http', 'https'])
+
     def __init__(self, auth=None):
         """
         """
@@ -21,13 +23,15 @@ class TornadoClient(BaseClient):
         """
         """
         req, resp = super(TornadoClient, self).request(req_and_resp, opt)
-        req.prepare(handle_files=True)
+
+        req.prepare(scheme=self.prepare_schemes(req).pop(), handle_files=True)
+        req._patch(opt)
 
         url = url_concat(req.url, req.query)
 
         rq = HTTPRequest(
             url=url,
-            method=req.method,
+            method=req.method.upper(),
             headers=req.header,
             body=req.data
             )
