@@ -1,16 +1,23 @@
 from __future__ import absolute_import
 from .getter import UrlGetter, LocalGetter
+from .primitives import SwaggerPrimitive
 from .spec.v1_2.parser import ResourceListContext
 from .spec.v2_0.parser import SwaggerContext
+from .spec.v2_0.objects import Operation
 from .scan import Scanner
 from .scanner import TypeReduce, CycleDetector
 from .scanner.v1_2 import Upgrade
-from .scanner.v2_0 import AssignParent, Resolve, PatchObject
-from pyswagger import utils, errs
+from .scanner.v2_0 import AssignParent, Resolve, PatchObject, YamlFixer
+from pyswagger import utils, errs, consts
 import inspect
 import base64
 import six
 import weakref
+import os
+import logging
+
+
+logger = logging.getLogger(__name__)
 
 
 class SwaggerApp(object):
@@ -26,13 +33,18 @@ class SwaggerApp(object):
         sc_path: ('/', '#/paths')
     }
 
-    def __init__(self, url=None, app_cache=None, url_load_hook=None):
+    def __init__(self, url=None, app_cache=None, url_load_hook=None, sep=consts.private.SCOPE_SEPARATOR, prim=None):
         """ constructor
 
         :param url str: url of swagger.json
         :param dict app_cache: a url map shared by SwaggerApp(s), mapping from url to SwaggerApp
         :param func url_load_hook: a way to redirect url to a accessible place. for self testing.
+        :param sep str: separator used by pyswager.utils.ScopeDict
+        :param prim pyswagger.primitives.SwaggerPrimitive: factory for primitives in Swagger.
         """
+
+        logger.info('init with url: {0}'.format(url))
+
         self.__root = None
         self.__raw = None
         self.__version = ''
@@ -51,7 +63,11 @@ class SwaggerApp(object):
         # all urls to load json would go through this hook
         self.__url_load_hook = url_load_hook
 
-        # TODO: allow init App-wised SCOPE_SEPARATOR
+        # allow init App-wised SCOPE_SEPARATOR
+        self.__sep = sep
+
+        # init a default SwaggerPrimitive as factory for primitives
+        self.__prim = prim if prim else SwaggerPrimitive()
 
     @property
     def root(self):
@@ -129,10 +145,20 @@ class SwaggerApp(object):
         """
         return self.__app_cache
 
-    def _load_json(self, url, getter=None, parser=None):
+    @property
+    def prim_factory(self):
+        """ primitive factory used by this app
+
+        :type: pyswagger.primitives.SwaggerPrimitive
+        """
+        return self.__prim
+
+    def _load_obj(self, url, getter=None, parser=None):
         """
         """
         if url in self.__app_cache:
+            logger.info('{0} hit cache'.format(url))
+
             # look into cache first
             return
 
@@ -142,16 +168,18 @@ class SwaggerApp(object):
 
             local_url = url if not self.__url_load_hook else self.__url_load_hook(url)
 
+            logger.info('{0} patch to {1}'.format(url, local_url))
+
             getter = UrlGetter
             p = six.moves.urllib.parse.urlparse(local_url)
             if p.scheme == 'file' and p.path:
-                getter = LocalGetter(p.path)
+                getter = LocalGetter(os.path.join(p.netloc, p.path))
 
-            if inspect.isclass(getter):
-                # default initialization is passing the url
-                # you can override this behavior by passing an
-                # initialized getter object.
-                getter = getter(local_url)
+        if inspect.isclass(getter):
+            # default initialization is passing the url
+            # you can override this behavior by passing an
+            # initialized getter object.
+            getter = getter(local_url)
 
         # get root document to check its swagger version.
         obj, _ = six.advance_iterator(getter)
@@ -173,6 +201,8 @@ class SwaggerApp(object):
         else:
             raise NotImplementedError('Unsupported Swagger Version: {0} from {1}'.format(version, url))
 
+        logger.info('version: {0}'.format(version))
+
         self.__app_cache[url] = weakref.proxy(self) # avoid circular reference
         self.__version = version
         self.__raw = tmp['_tmp_']
@@ -213,7 +243,7 @@ class SwaggerApp(object):
         self.validate(strict=strict)
 
         if self.version == '1.2':
-            converter = Upgrade()
+            converter = Upgrade(self.__sep)
             s.scan(root=self.raw, route=[converter])
             obj = converter.swagger
 
@@ -223,6 +253,7 @@ class SwaggerApp(object):
 
             self.__root = obj
         elif self.version == '2.0':
+            s.scan(root=self.raw, route=[YamlFixer()], leaves=[Operation])
             self.__root = self.raw
         else:
             raise NotImplementedError('Unsupported Version: {0}'.format(self.__version))
@@ -235,7 +266,7 @@ class SwaggerApp(object):
         s.scan(root=self.__root, route=[PatchObject()])
 
     @classmethod
-    def load(kls, url, getter=None, parser=None, app_cache=None, url_load_hook=None):
+    def load(kls, url, getter=None, parser=None, app_cache=None, url_load_hook=None, sep=consts.private.SCOPE_SEPARATOR, prim=None):
         """ load json as a raw SwaggerApp
 
         :param str url: url of path of Swagger API definition
@@ -245,16 +276,20 @@ class SwaggerApp(object):
         :type parser: pyswagger.base.Context
         :param dict app_cache: the cache shared by related SwaggerApp
         :param func url_load_hook: hook to patch the url to load json
+        :param str sep: scope-separater used in this SwaggerApp
+        :param prim pyswager.primitives.SwaggerPrimitive: factory for primitives in Swagger
         :return: the created SwaggerApp object
         :rtype: SwaggerApp
         :raises ValueError: if url is wrong
         :raises NotImplementedError: the swagger version is not supported.
         """
 
-        url = utils.normalize_url(url)
-        app = kls(url, app_cache, url_load_hook)
+        logger.info('load with [{0}]'.format(url))
 
-        app._load_json(url, getter, parser)
+        url = utils.normalize_url(url)
+        app = kls(url, app_cache=app_cache, url_load_hook=url_load_hook, sep=sep, prim=prim)
+
+        app._load_obj(url, getter, parser)
 
         # update schem if any
         p = six.moves.urllib.parse.urlparse(url)
@@ -287,17 +322,20 @@ class SwaggerApp(object):
 
         # reducer for Operation 
         s = Scanner(self)
-        tr = TypeReduce()
+        tr = TypeReduce(self.__sep)
         cy = CycleDetector()
         s.scan(root=self.__root, route=[tr, cy])
 
         # 'op' -- shortcut for Operation with tag and operaionId
         self.__op = utils.ScopeDict(tr.op)
         # 'm' -- shortcut for model in Swagger 1.2
-        if hasattr(self.__root, 'definitions'):
+        if hasattr(self.__root, 'definitions') and self.__root.definitions != None:
             self.__m = utils.ScopeDict(self.__root.definitions)
         else:
             self.__m = utils.ScopeDict({})
+        # update scope-separater
+        self.__m.sep = self.__sep
+        self.__op.sep = self.__sep
 
         # cycle detection
         if len(cy.cycles['schema']) > 0 and strict:
@@ -335,6 +373,8 @@ class SwaggerApp(object):
         :raises ValueError: if path is not valid
         """
 
+        logger.info('resolving: [{0}]'.format(jref))
+
         if jref == None or len(jref) == 0:
             raise ValueError('Empty Path is not allowed')
 
@@ -377,6 +417,16 @@ class SwaggerApp(object):
         else:
             return self.resolve(utils.jp_compose(p, base=b[1]))
 
+    def dump(self):
+        """ dump into Swagger Document
+
+        :rtype: dict
+        :return: dict representation of Swagger
+        """
+        return self.root.dump()
+
+
+
 class SwaggerSecurity(object):
     """ security handler
     """
@@ -429,15 +479,19 @@ class SwaggerSecurity(object):
         if not req._security:
             return req
 
-        for k, v in six.iteritems(req._security):
-            if not k in self.__info:
-                continue
+        for s in req._security:
+            for k, v in six.iteritems(s):
+                if not k in self.__info:
+                    logger.info('missing: [{0}]'.format(k))
+                    continue
 
-            header, cred = self.__info[k]
-            if header:
-                req._p['header'].update(cred)
-            else:
-                utils.nv_tuple_list_replace(req._p['query'], utils.get_dict_as_tuple(cred))
+                logger.info('applying: [{0}]'.format(k))
+
+                header, cred = self.__info[k]
+                if header:
+                    req._p['header'].update(cred)
+                else:
+                    utils.nv_tuple_list_replace(req._p['query'], utils.get_dict_as_tuple(cred))
 
         return req
 
@@ -498,6 +552,13 @@ class BaseClient(object):
         """
         req, resp = req_and_resp
 
+        # dump info for debugging
+        logger.info('request.url: {0}'.format(req.url))
+        logger.info('request.header: {0}'.format(req.header))
+        logger.info('request.query: {0}'.format(req.query))
+        logger.info('request.file: {0}'.format(req.files))
+        logger.info('request.schemes: {0}'.format(req.schemes))
+
         # apply authorizations
         if self.__security:
             self.__security(req) 
