@@ -1,8 +1,13 @@
 from __future__ import absolute_import
-from pyswagger import const
+from .consts import private
 import json
+import yaml
 import six
 import os
+import logging
+
+
+logger = logging.getLogger(__name__)
 
 
 class Getter(six.Iterator):
@@ -24,13 +29,21 @@ class Getter(six.Iterator):
 
         path, name = self.urls.pop(0)
         obj = self.load(path)
-        if isinstance(obj, six.string_types):
-            obj = json.loads(obj)
-        elif isinstance(obj, six.binary_type):
-            obj = json.loads(obj.decode('utf-8'))
-        else:
+
+        # make sure data is string type
+        if isinstance(obj, six.binary_type):
+            obj = obj.decode('utf-8')
+        elif not isinstance(obj, six.string_types):
             raise ValueError('Unknown types: [{0}]'.format(str(type(obj))))
 
+        # a very simple logic to distinguish json and yaml
+        if obj.startswith('{'):
+            obj = json.loads(obj)
+        elif obj.startswith('---'):
+            obj = yaml.load(obj)
+        else:
+            raise Exception('Unknown format startswith {0} ...'.format(obj[:10]))
+
         # find urls to retrieve from resource listing file
         if name == '':
             urls = self.__find_urls(obj)
@@ -57,12 +70,13 @@ class Getter(six.Iterator):
         :rtype: a list of str
         """
         urls = []
-        if const.SCHEMA_APIS in obj:
-            if isinstance(obj[const.SCHEMA_APIS], list):
-                for api in obj[const.SCHEMA_APIS]:
-                    urls.append(api[const.SCHEMA_PATH])
+        if private.SCHEMA_APIS in obj:
+            # This is a Swagger 1.2 spec, need to load subsequent resource files.
+            if isinstance(obj[private.SCHEMA_APIS], list):
+                for api in obj[private.SCHEMA_APIS]:
+                    urls.append(api[private.SCHEMA_PATH])
             else:
-                raise TypeError('Invalid type of apis: ' + type(obj[const.SCHEMA_APIS]))
+                raise TypeError('Invalid type of apis: ' + type(obj[private.SCHEMA_APIS]))
 
         return urls
 
@@ -73,7 +87,7 @@ class LocalGetter(Getter):
     def __init__(self, path):
         super(LocalGetter, self).__init__(path)
 
-        for n in const.SWAGGER_FILE_NAMES:
+        for n in private.SWAGGER_FILE_NAMES:
             if self.base_path.endswith(n):
                 self.base_path = os.path.dirname(self.base_path)
                 self.urls = [(path, '')]
@@ -84,17 +98,44 @@ class LocalGetter(Getter):
                     self.urls = [(p, '')]
                     break
         else:
-            raise ValueError('Unable to locate resource file: [{0}]'.format(path))
+            # there is no file matched predefined file name:
+            # - resource_list.json (1.2)
+            # - swagger.json       (2.0)
+            # in this case, we will locate them in this way:
+            # - when 'path' points to a specific file, and its
+            #   extension is either 'json' or 'yaml'.
+            _, ext = os.path.splitext(path)
+            for e in [private.FILE_EXT_JSON, private.FILE_EXT_YAML]:
+                if ext.endswith(e):
+                    self.base_path = os.path.dirname(path)
+                    self.urls=[(path, '')]
+                    break
+            else:
+                raise ValueError('Unable to locate resource file: [{0}]'.format(path))
 
     def load(self, path):
         ret = None
-        # make sure we get .json files
-        if not path.endswith(const.RESOURCE_FILE_EXT):
-            path = path + '.' + const.RESOURCE_FILE_EXT
+
+        logger.info('to load: [{0}]'.format(path))
+
+        # try to get extension from Getter.base_path
+        _, ext = os.path.splitext(self.base_path)
+        # try to get extension from path
+        _, ext = os.path.splitext(path) if ext == '' else (None, ext)
+        # .json is default extension to try
+        ext = '.json' if ext == '' else ext
+        # make sure we get .json or .yaml files
+        if not path.endswith(ext):
+            path = path + ext
+
+        # trim the leading slash, which is invalid on Windows
+        if os.name == 'nt' and path.startswith('/'):
+            path = path[1:]
+
+        logger.info('final path to load: [{0}]'.format(path))
 
         with open(path, 'r') as f:
             ret = f.read()
-
         return ret
 
 
@@ -108,6 +149,9 @@ class UrlGetter(Getter):
         self.urls = [(path, '')]
 
     def load(self, path):
+
+        logger.info('to load: [{0}]'.format(path))
+
         ret = f = None
         try:
             f = six.moves.urllib.request.urlopen(path)
