@@ -1,11 +1,16 @@
 from __future__ import absolute_import
-from .primitives import PrimJSONEncoder
+from .primitives.comm import PrimJSONEncoder
 from .utils import deref
 from pyswagger import errs
 from uuid import uuid4
 import six
 import json
 import io, codecs
+import collections
+import logging
+
+
+logger = logging.getLogger(__name__)
 
 
 class SwaggerRequest(object):
@@ -30,6 +35,7 @@ class SwaggerRequest(object):
         self.__op = op
         self.__p = params
         self.__url = self.__op.url
+        self.__path = self.__op.path
         self.__header = {}
 
         # update 'accept' header section
@@ -134,6 +140,8 @@ class SwaggerRequest(object):
                 (scheme, opt_netloc, path, params, query, fragment)
                 )
 
+            logger.info('patching url: [{0}]'.format(self.__url))
+
     def prepare(self, scheme='http', handle_files=True, encoding='utf-8'):
         """ make this request ready for Clients
 
@@ -142,6 +150,9 @@ class SwaggerRequest(object):
         :rtype: SwaggerRequest
         """
 
+        # combine path parameters into path
+        self.__path = self.__path.format(**self.__p['path'])
+
         # combine path parameters into url
         self.__url = ''.join([scheme, '://', self.__url.format(**self.__p['path'])])
 
@@ -172,7 +183,7 @@ class SwaggerRequest(object):
             self.__header.update({'Content-Type': content_type})
 
         return self
-    
+
 
     @property
     def url(self):
@@ -182,6 +193,22 @@ class SwaggerRequest(object):
         """
         return self.__url
 
+    @property
+    def path(self):
+        """ path of this request
+
+        :type: str
+        """
+        return self.__path
+
+    @property
+    def base_path(self):
+        """ base path of this request
+
+        :type: str
+        """
+        return self.__op.base_path
+
     @property
     def query(self):
         """ query part of this request
@@ -228,7 +255,7 @@ class SwaggerRequest(object):
 
         :type: list of str
         """
-        return self.__op.schemes
+        return self.__op.cached_schemes
 
     @property
     def _p(self):
@@ -262,6 +289,15 @@ class SwaggerResponse(object):
         self.__status = None 
         self.__header = {}
 
+    def _convert_header(self, resp, k, v):
+        if resp and resp.headers and k in resp.headers:
+            v = resp.headers[k]._prim_(v, self.__op._prim_factory)
+
+        if k in self.__header:
+            self.__header[k].append(v)
+        else:
+            self.__header[k] = [v]
+
     def apply_with(self, status=None, raw=None, header=None):
         """ update header, status code, raw datum, ...etc
 
@@ -275,28 +311,27 @@ class SwaggerResponse(object):
         if status != None:
             self.__status = status
 
-        r = deref(self.__op.responses.get(str(self.__status), None))
-        r = deref(self.__op.responses.get('default', None)) if r == None else r
+        r = (deref(self.__op.responses.get(str(self.__status), None)) or
+             deref(self.__op.responses.get('default', None)))
 
         if raw != None:
+            # update 'raw'
+            self.__raw = raw
+
             if self.__status == None:
                 raise Exception('Update status code before assigning raw data')
 
-            if r.schema:
-                self.__raw = raw
-
+            if r and r.schema:
                 # update data from Opeartion if succeed else from responseMessage.responseModel
-                self.__data = r.schema._prim_(self.raw)
+                self.__data = r.schema._prim_(self.raw, self.__op._prim_factory)
 
         if header != None:
-            for k, v in six.iteritems(header):
-                if k in r.headers:
-                    v = r.headers[k]._prim_(v)
-
-                if k in self.__header:
-                    self.__header[k].extend(v)
-                else:
-                    self.__header[k] = [v]
+            if isinstance(header, (collections.Mapping, collections.MutableMapping)):
+                for k, v in six.iteritems(header):
+                    self._convert_header(r, k, v)
+            else:
+                for k, v in header:
+                    self._convert_header(r, k, v)
 
         return self
 
